@@ -73,14 +73,15 @@ static void save_wm_settings (void);
 static void save_lxsession_settings (void);
 static void save_gsettings (void);
 static void save_xsettings (void);
+static void save_environment (void);
 static void save_labwc_to_settings (void);
 static int is_dark (void);
 static gboolean restore_theme (gpointer data);
-static void on_theme_colour_set (GtkColorChooser* btn, gpointer ptr);
-static void on_theme_textcolour_set (GtkColorChooser* btn, gpointer ptr);
-static void on_theme_font_set (GtkFontChooser* btn, gpointer ptr);
-static void on_theme_dark_set (GtkRadioButton* btn, gpointer ptr);
-static void on_theme_cursor_size_set (GtkComboBox* btn, gpointer ptr);
+static void on_theme_colour_set (GtkColorChooser *btn, gpointer ptr);
+static void on_theme_textcolour_set (GtkColorChooser *btn, gpointer ptr);
+static void on_theme_font_set (GtkFontChooser *btn, gpointer ptr);
+static void on_theme_dark_set (GtkRadioButton *btn, gpointer ptr);
+static void on_theme_cursor_size_set (GtkComboBox *btn, gpointer ptr);
 
 /*----------------------------------------------------------------------------*/
 /* Function definitions                                                       */
@@ -174,7 +175,7 @@ static void add_or_amend (const char *conffile, const char *block, const char *p
 
 char *lxsession_file (gboolean global)
 {
-    return g_build_filename (global ? "/etc/xdg" : g_get_user_config_dir (), "lxsession", session (), "desktop.conf", NULL);
+    return g_build_filename (global ? "/etc/xdg" : g_get_user_config_dir (), "lxsession", "rpd-x", "desktop.conf", NULL);
 }
 
 char *xsettings_file (gboolean global)
@@ -184,13 +185,7 @@ char *xsettings_file (gboolean global)
 
 char *openbox_file (void)
 {
-    const char *session_name = session ();
-    char *lc_sess = g_ascii_strdown (session_name, -1);
-    char *fname = g_strconcat (lc_sess, "-rc.xml", NULL);
-    char *path = g_build_filename (g_get_user_config_dir (), "openbox", fname, NULL);
-    g_free (lc_sess);
-    g_free (fname);
-    return path;
+    return g_build_filename (g_get_user_config_dir (), "openbox", "rpd-rc.xml", NULL);
 }
 
 static char *labwc_file (void)
@@ -322,14 +317,14 @@ static void load_gtk3_settings (void)
     cur_conf.darkmode = (is_dark () == 1) ? TRUE : FALSE;
 
     cur_conf.scrollbar_width = 13;
-    user_config_file = g_build_filename (g_get_user_data_dir (), "themes", cur_conf.darkmode ? DEFAULT_THEME_DARK : DEFAULT_THEME, "gtk-3.0/gtk.css", NULL);
+    user_config_file = g_build_filename (g_get_user_data_dir (), "themes", theme_name (cur_conf.darkmode), "gtk-3.0/gtk.css", NULL);
     if (!vsystem ("grep -q \"min-width: 17px\" %s 2> /dev/null", user_config_file)) cur_conf.scrollbar_width = 17;
     g_free (user_config_file);
 
     for (dark = 0; dark < 2; dark++)
     {
-        sys_config_file = g_build_filename ("/usr/share/themes", dark ? DEFAULT_THEME_DARK : DEFAULT_THEME, "gtk-3.0/!(*-dark).css", NULL);
-        user_config_file = g_build_filename (g_get_user_data_dir (), "themes", dark ? DEFAULT_THEME_DARK : DEFAULT_THEME, "gtk-3.0/*.css", NULL);
+        sys_config_file = g_build_filename ("/usr/share/themes", theme_name (dark), "gtk-3.0/!(*-dark).css", NULL);
+        user_config_file = g_build_filename (g_get_user_data_dir (), "themes", theme_name (dark), "gtk-3.0/*.css", NULL);
 
         cmdbuf = g_strdup_printf ("grep -hPo '(?<=@define-color\\stheme_selected_bg_color\\s)#[0-9A-Fa-f]{6}' %s 2> /dev/null", user_config_file);
         res = get_string (cmdbuf);
@@ -390,7 +385,7 @@ static void load_gtk3_settings (void)
 
 static void save_wm_settings (void)
 {
-    char *user_config_file, *font, *cptr;
+    char *user_config_file, *theme, *font, *cptr;
     int count, size;
     const gchar *weight = NULL, *style = NULL;
     char buf[10];
@@ -538,25 +533,21 @@ static void save_wm_settings (void)
     }
     xmlXPathFreeObject (xpathObj);
 
+    theme = g_strdup_printf ("%s%s", theme_name (cur_conf.darkmode), cur_conf.scrollbar_width >= 17 ? "_l" : "");
     xpathObj = xmlXPathEvalExpression (XC ("/*[local-name()='openbox_config']/*[local-name()='theme']/*[local-name()='name']"), xpathCtx);
     if (xmlXPathNodeSetIsEmpty (xpathObj->nodesetval))
     {
         xmlXPathFreeObject (xpathObj);
         xpathObj = xmlXPathEvalExpression (XC ("/*[local-name()='openbox_config']/*[local-name()='theme']"), xpathCtx);
         cur_node = xpathObj->nodesetval->nodeTab[0];
-        if (cur_conf.scrollbar_width >= 17)
-            xmlNewChild (cur_node, NULL, XC ("name"), cur_conf.darkmode ? XC (DEFAULT_THEME_DARK_L) : XC (DEFAULT_THEME_L));
-        else
-            xmlNewChild (cur_node, NULL, XC ("name"), cur_conf.darkmode ? XC (DEFAULT_THEME_DARK) : XC (DEFAULT_THEME));
+        xmlNewChild (cur_node, NULL, XC ("name"), XC (theme));
     }
     else
     {
         cur_node = xpathObj->nodesetval->nodeTab[0];
-        if (cur_conf.scrollbar_width >= 17)
-            xmlNodeSetContent (cur_node, cur_conf.darkmode ? XC (DEFAULT_THEME_DARK_L) : XC (DEFAULT_THEME_L));
-        else
-            xmlNodeSetContent (cur_node, cur_conf.darkmode ? XC (DEFAULT_THEME_DARK) : XC (DEFAULT_THEME));
+        xmlNodeSetContent (cur_node, XC (theme));
     }
+    g_free (theme);
 
     if (wm == WM_LABWC) save_labwc_to_settings ();
     else
@@ -635,7 +626,7 @@ static void save_lxsession_settings (void)
     kf = g_key_file_new ();
     g_key_file_load_from_file (kf, user_config_file, G_KEY_FILE_KEEP_COMMENTS | G_KEY_FILE_KEEP_TRANSLATIONS, NULL);
 
-    g_key_file_set_string (kf, "GTK", "sNet/ThemeName", TEMP_THEME);
+    g_key_file_set_string (kf, "GTK", "sNet/ThemeName", theme_name (TEMP));
 
     // update changed values in the key file
     ctheme = rgba_to_gdk_color_string (&cur_conf.theme_colour[cur_conf.darkmode]);
@@ -730,10 +721,10 @@ void save_gtk3_settings (void)
     g_free (user_config_file);
 
     // create a temp theme to switch to
-    link1 = g_build_filename (g_get_user_data_dir (), "themes", TEMP_THEME, NULL);
+    link1 = g_build_filename (g_get_user_data_dir (), "themes", theme_name (TEMP), NULL);
     if (!g_file_test (link1, G_FILE_TEST_IS_DIR))
     {
-        link2 = g_build_filename (g_get_user_data_dir (), "themes", DEFAULT_THEME, NULL);
+        link2 = g_build_filename (g_get_user_data_dir (), "themes", theme_name (LIGHT), NULL);
         symlink (link2, link1);
         g_free (link2);
     }
@@ -747,12 +738,12 @@ void save_gtk3_settings (void)
         cstrbf = rgba_to_gdk_color_string (&cur_conf.bartext_colour[dark]);
 
         // construct the file path
-        user_config_file = g_build_filename (g_get_user_data_dir (), "themes", dark ? DEFAULT_THEME_DARK : DEFAULT_THEME, "gtk-3.0/gtk.css", NULL);
+        user_config_file = g_build_filename (g_get_user_data_dir (), "themes", theme_name (dark), "gtk-3.0/gtk.css", NULL);
         check_directory (user_config_file);
 
         if (!g_file_test (user_config_file, G_FILE_TEST_IS_REGULAR))
         {
-            vsystem ("echo '@import url(\"/usr/share/themes/%s/gtk-3.0/gtk.css\");' >> %s", dark ? DEFAULT_THEME_DARK : DEFAULT_THEME, user_config_file);
+            vsystem ("echo '@import url(\"/usr/share/themes/%s/gtk-3.0/gtk.css\");' >> %s", theme_name (dark), user_config_file);
             vsystem ("echo '@define-color theme_selected_bg_color %s;' >> %s", cstrb, user_config_file);
             vsystem ("echo '@define-color theme_selected_fg_color %s;' >> %s", cstrf, user_config_file);
             vsystem ("echo '@define-color bar_bg_color %s;' >> %s", cstrbb, user_config_file);
@@ -869,14 +860,39 @@ static void save_xsettings (void)
     g_free (user_config_file);
 }
 
+static void save_environment (void)
+{
+    char *user_config_file;
+
+    // construct the file path
+    user_config_file = g_build_filename (g_get_user_config_dir (), "labwc", "environment", NULL);
+    check_directory (user_config_file);
+
+    if (!g_file_test (user_config_file, G_FILE_TEST_IS_REGULAR))
+    {
+        vsystem ("echo 'XCURSOR_SIZE=%d' >> %s", cur_conf.cursor_size, user_config_file);
+        g_free (user_config_file);
+        return;
+    }
+
+    // amend entry already in file, or add if not present
+    if (vsystem ("grep -q XCURSOR_SIZE %s\n", user_config_file))
+        vsystem ("echo 'XCURSOR_SIZE=%d' >> %s", cur_conf.cursor_size, user_config_file);
+    else
+        vsystem ("sed -i s/'XCURSOR_SIZE.*'/'XCURSOR_SIZE=%d'/g %s", cur_conf.cursor_size, user_config_file);
+
+    g_free (user_config_file);
+}
+
 void save_session_settings (void)
 {
-    set_theme (TEMP_THEME);
+    set_theme (theme_name (TEMP));
     if (wm == WM_OPENBOX) save_lxsession_settings ();
     else 
     {
         save_xsettings ();
         save_gsettings ();
+        save_environment ();
     }
     save_wm_settings ();
 }
@@ -919,6 +935,26 @@ static void save_labwc_to_settings (void)
     g_free (user_config_file);
 }
 
+void save_greeter_settings (void)
+{
+    GKeyFile *kf;
+    char *str;
+    gsize len;
+
+    kf = g_key_file_new ();
+    g_key_file_load_from_file (kf, "/etc/lightdm/pi-greeter.conf", G_KEY_FILE_KEEP_COMMENTS | G_KEY_FILE_KEEP_TRANSLATIONS, NULL);
+
+    g_key_file_set_value (kf, "greeter", "wallpaper", cur_conf.darkmode ? "/usr/share/rpd-wallpaper/RPiSystem_dark.png" : "/usr/share/rpd-wallpaper/RPiSystem.png");
+    g_key_file_set_value (kf, "greeter", "gtk-theme-name", cur_conf.darkmode ? "PiXonyx" : "PiXtrix");
+    g_key_file_set_value (kf, "greeter", "gtk-font-name", cur_conf.desktop_font);
+
+    str = g_key_file_to_data (kf, &len, NULL);
+    g_file_set_contents (GREETER_TMP, str, len, NULL);
+
+    g_free (str);
+    g_key_file_free (kf);
+}
+
 void save_qt_settings (void)
 {
     char *user_config_file, *str;
@@ -1126,17 +1162,17 @@ static int is_dark (void)
 {
     int res;
 
-    char *config_file = g_build_filename ("/usr/share/themes", DEFAULT_THEME_DARK, "gtk-3.0/gtk.css", NULL);
+    char *config_file = g_build_filename ("/usr/share/themes", theme_name (DARK), "gtk-3.0/gtk.css", NULL);
     if (access (config_file, F_OK)) return -1;
     g_free (config_file);
 
     if (wm == WM_OPENBOX)
     {
         char *user_config_file = lxsession_file (FALSE);
-        res = vsystem ("grep sNet/ThemeName %s | grep -q %s", user_config_file, DEFAULT_THEME_DARK);
+        res = vsystem ("grep sNet/ThemeName %s | grep -q %s", user_config_file, theme_name (DARK));
         g_free (user_config_file);
     }
-    else res = vsystem ("gsettings get org.gnome.desktop.interface gtk-theme | grep -q %s", DEFAULT_THEME_DARK);
+    else res = vsystem ("gsettings get org.gnome.desktop.interface gtk-theme | grep -q %s", theme_name (DARK));
 
     if (!res) return 1;
     else return 0;
@@ -1145,7 +1181,7 @@ static int is_dark (void)
 static gboolean restore_theme (gpointer data)
 {
     /* Resets the theme to the default, causing it to take effect */
-    set_theme (cur_conf.darkmode ? DEFAULT_THEME_DARK : DEFAULT_THEME);
+    set_theme (theme_name (cur_conf.darkmode));
     if (data) gtk_main_quit ();
     return FALSE;
 }
@@ -1188,7 +1224,7 @@ void set_system_controls (void)
 /* Control handlers                                                           */
 /*----------------------------------------------------------------------------*/
 
-static void on_theme_colour_set (GtkColorChooser* btn, gpointer ptr)
+static void on_theme_colour_set (GtkColorChooser *btn, gpointer ptr)
 {
     gtk_color_chooser_get_rgba (btn, &cur_conf.theme_colour[cur_conf.darkmode]);
 
@@ -1199,7 +1235,7 @@ static void on_theme_colour_set (GtkColorChooser* btn, gpointer ptr)
     reload_theme (FALSE);
 }
 
-static void on_theme_textcolour_set (GtkColorChooser* btn, gpointer ptr)
+static void on_theme_textcolour_set (GtkColorChooser *btn, gpointer ptr)
 {
     gtk_color_chooser_get_rgba (btn, &cur_conf.themetext_colour[cur_conf.darkmode]);
 
@@ -1210,7 +1246,7 @@ static void on_theme_textcolour_set (GtkColorChooser* btn, gpointer ptr)
     reload_theme (FALSE);
 }
 
-static void on_theme_font_set (GtkFontChooser* btn, gpointer ptr)
+static void on_theme_font_set (GtkFontChooser *btn, gpointer ptr)
 {
     int i;
     PangoFontDescription *font_desc = gtk_font_chooser_get_font_desc (btn);
@@ -1234,6 +1270,7 @@ static void on_theme_font_set (GtkFontChooser* btn, gpointer ptr)
     for (i = 0; i < ndesks; i++)
         save_pcman_settings (i);
     save_qt_settings ();
+    save_greeter_settings ();
 
     reload_session ();
     reload_panel ();
@@ -1241,7 +1278,7 @@ static void on_theme_font_set (GtkFontChooser* btn, gpointer ptr)
     reload_theme (FALSE);
 }
 
-static void on_theme_dark_set (GtkRadioButton* btn, gpointer ptr)
+static void on_theme_dark_set (GtkRadioButton *btn, gpointer ptr)
 {
     if (!system ("pgrep geany > /dev/null"))
     {
@@ -1273,12 +1310,13 @@ static void on_theme_dark_set (GtkRadioButton* btn, gpointer ptr)
     save_session_settings ();
     save_gtk3_settings ();
     save_app_settings ();
+    save_greeter_settings ();
 
     reload_session ();
     reload_theme (FALSE);
 }
 
-static void on_theme_cursor_size_set (GtkComboBox* btn, gpointer ptr)
+static void on_theme_cursor_size_set (GtkComboBox *btn, gpointer ptr)
 {
     gint val = gtk_combo_box_get_active (btn);
     switch (val)
